"""Llamabot Zotero CLI."""
import json
from pathlib import Path

import typer
from dotenv import load_dotenv
from rich.progress import Progress, SpinnerColumn, TextColumn

from llamabot import QueryBot
from llamabot.prompt_library.zotero import get_key, retrieverbot_sysprompt
from llamabot.zotero.library import ZoteroLibrary
from llamabot.zotero.utils import load_zotero

from .utils import configure_environment_variable

load_dotenv()

app = typer.Typer()

ZOTERO_JSON_PATH = Path.home() / ".llamabot/zotero/zotero_index.json"
ZOTERO_JSON_PATH.parent.mkdir(parents=True, exist_ok=True)
progress = Progress(
    SpinnerColumn(),
    TextColumn("[progress.description]{task.description}"),
    transient=True,
)


@app.command()
def configure(
    library_id: str = typer.Option(..., prompt=True),
    api_key: str = typer.Option(..., prompt=True),
    library_type: str = typer.Option(default="user", prompt=True),
):
    """Configure Llamabot Zotero CLI environment variables.

    :param library_id: Zotero library ID
    :param api_key: Zotero API key
    :param library_type: Zotero library type
    """
    configure_environment_variable("ZOTERO_LIBRARY_ID", library_id)
    configure_environment_variable("ZOTERO_API_KEY", api_key)
    configure_environment_variable("ZOTERO_LIBRARY_TYPE", library_type)


@app.command()
def sync():
    """Sync Zotero items to a local JSON file."""

    zot = load_zotero()
    with progress:
        progress.add_task("Syncing Zotero items...")
        items = zot.everything(zot.items())

    with open(ZOTERO_JSON_PATH, "w+") as f:
        for item in items:
            f.write(json.dumps(item) + "\n")


@app.command()
def chat_paper(query: str = ""):
    """Chat with a paper.

    :param query: A paper to search for, whether by title, author, or other metadata.
    """
    typer.echo("Llamabot Zotero Chatbot initializing...")
    typer.echo("Use Ctrl+C to exit.")

    library = ZoteroLibrary()

    with progress:
        progress.add_task("Embedding Zotero library...")
        retrieverbot = QueryBot(
            retrieverbot_sysprompt(),
            doc_path=ZOTERO_JSON_PATH,
        )
    response = retrieverbot(get_key(query))
    paper_key = json.loads(response.content)["key"]
    typer.echo(f"Retrieved key: {paper_key}")
    typer.echo(f"Paper title: {library[paper_key]['data.title']}")

    # Retrieve paper from library
    with progress:
        progress.add_task("Downloading paper...")
        fpath = library[paper_key].download_pdf(Path("/tmp"))
    typer.echo(f"Downloaded paper to {fpath}")

    with progress:
        progress.add_task("Embedding paper and initializing bot...")
        docbot = QueryBot(
            "You are an expert in answering questions about a paper.",
            doc_path=fpath,
            temperature=0.3,
        )

    while True:
        query = input("Ask me a question: ")
        response = docbot(query)
        print("\n\n")



"""CLI utility functions."""

from pathlib import Path


def get_valid_input(prompt, valid_inputs=("y", "n")):
    """
    This function prompts the user for input and validates it.

    .. code-block:: python

        user_choice = get_valid_input("Enter 'y' for yes or 'n' for no: ")

    :param prompt: The prompt to display to the user.
    :param valid_inputs: A tuple of valid inputs.
    :return: The validated user input, either 'y' or 'n'.
    """
    msg = f"{prompt} {valid_inputs}"
    while True:
        user_input = input(msg).lower()
        if user_input in valid_inputs:
            return user_input
        else:
            print(f"Invalid input. Please enter one of { '/'.join(valid_inputs) }.")


def configure_environment_variable(env_var: str, env_value: str):
    """Configure environment variable in ~/.llamabotrc.

    :param env_var: The name of the environment variable.
        Will be converted to upper case.
    :param env_value: The API key.
    """
    config_file = Path.home() / ".llamabot/.llamabotrc"
    env_var_line = f'export {env_var.upper()}="{env_value}"'

    if config_file.exists():
        with open(config_file, "r") as file:
            content = file.readlines()

        found_env_var = False
        with open(config_file, "w") as file:
            for line in content:
                if f"export {env_var.upper()}=" in line:
                    print(f"found env var line {env_var_line}")
                    file.write(env_var_line + "\n")
                    found_env_var = True
                else:
                    file.write(line)

        if not found_env_var:
            with open(config_file, "a") as file:
                file.write(env_var_line + "\n")
    else:
        with open(config_file, "w") as file:
            file.write(env_var_line + "\n")



"""
This module provides a top-level command line interface for interacting with apps and python.

Classes:
    - None

Exceptions:
    - None

Functions:
    - None

Objects:
    - app: A Typer instance for managing command line interface.

Modules:
    - apps: A module for managing apps-related commands.
    - python: A module for managing python-related commands.
"""

import typer

from . import apps, git, python, tutorial, zotero
from .utils import configure_environment_variable

app = typer.Typer()
app.add_typer(apps.app, name="apps")
app.add_typer(python.app, name="python")
app.add_typer(git.gitapp, name="git")
app.add_typer(tutorial.app, name="tutorial")
app.add_typer(zotero.app, name="zotero")


@app.command()
def configure(
    api_key: str = typer.Option(
        ..., prompt=True, hide_input=True, confirmation_prompt=True
    )
) -> None:
    """
    Configure the API key for llamabot.

    .. code-block:: python

        configure(api_key="your_api_key_here")

    :param api_key: The API key to be used for authentication.
    """
    configure_environment_variable(env_var="OPENAI_API_KEY", env_value=api_key)


@app.command()
def version():
    """
    Print the version of llamabot.
    """
    from llamabot.version import version

    typer.echo(version)


if __name__ == "__main__":
    app()



"""Zotero library wrappers."""
import json
from dataclasses import dataclass
from pathlib import Path

from .utils import load_zotero


@dataclass
class ZoteroLibrary:
    """Zotero library."""

    library_path: Path = Path.home() / ".llamabot/zotero/zotero_index.json"

    def __post_init__(self):
        """Post-initialization hook"""
        self.library = [json.loads(line) for line in open(self.library_path, "r")]

    def __getitem__(self, key):
        """Get item by key.

        :param key: Key to get.
        :return: Value for key.
        """
        return ZoteroItem([i for i in self.library if i["key"] == key][0], library=self)


@dataclass
class ZoteroItem:
    """Zotero item."""

    info: dict
    library: ZoteroLibrary

    def __getitem__(self, key):
        """Get item by key.

        :param key: Key to get.
        :return: Value for key.
        """
        # Key should be a string that is dot-delimited.
        # we split the string into a list of keys
        # Then we access the keys in order.

        keys = key.split(".")
        value = self.info
        for k in keys:
            value = value[k]
        return value

    def pdf(self):
        """Get the PDF entry for this item.

        :return: PDF entry.
        """
        pdf_url = self["links.attachment.href"]
        pdf_key = pdf_url.split("/")[-1]
        pdf_entry = self.library[pdf_key]
        return pdf_entry

    def download_pdf(self, directory: Path) -> Path:
        """Download the PDF for this item.

        :param directory: Directory to download the PDF to.
        :return: Path to the downloaded PDF.
        """
        pdf = self.pdf()
        zot = load_zotero()
        fpath = directory / f"{pdf['key']}.pdf"
        with open(fpath, "wb") as f:
            f.write(zot.file(pdf["key"]))
        return fpath
